/*
 * Copyright (C) 2004   Derrick Moser
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include "Composer.h"

bool Composer::parse(bool force)
{
  int n = m_input.getSize();
  if(n == 0)
    return false;

  const int *b = m_input.getBuffer();
  int s = m_parser.parse(b, n, force);
  if(s == Success)
  {
    m_output.append(m_parser.getResult());
    m_input.removeFirst(m_parser.getLength());
    return true;
  }
  else if(s == Failed)
  {
    m_output.append(b[0]);
    m_input.removeFirst(1);
    return true;
  }

  return false;
}

bool Composer::isEmpty() const
{
  return m_input.getSize() == 0;
}

void Composer::clear()
{
  m_input.clear();
}

bool Composer::append(int c)
{
  m_input.append(c);
  while(parse(false))
    ;
}

void Composer::backspace()
{
  if(m_input.getSize())
    m_input.removeLast(1);
}

bool Composer::accept()
{
  int n = m_input.getSize();
  const int *b = m_input.getBuffer();
  for(int i = 0; i < n; ++i)
    m_output.append(b[i]);
  m_input.clear();
}

bool Composer::convert()
{
  while(parse(true))
    ;
}
