/*
 * Copyright (C) 2004   Derrick Moser
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include "Parser.h"

Parser::Parser(int npatterns, Pattern *patterns) :
  m_npatterns(npatterns), m_patterns(patterns)
{
}

int Parser::parse(const int *s, int len, int max_pattern, bool force)
{
  int status = Failed;
  int length = -1;
  int result;

  for(int i = 0; i < m_npatterns; ++i)
  {
    const char *pattern = m_patterns[i].m_pattern;
    int j = 0;
    while(j < len && pattern[j] && (unsigned char)pattern[j] == s[j])
      ++j;
    if(j > max_pattern)
    {
      if(!force)
      {
        status = Incomplete;
        break;
      }
    }
    else if(j <= max_pattern && j > length && !pattern[j])
    {
      status = Success;
      result = m_patterns[i].m_result;
      length = j;
    }
    else if(!force && j >= len)
    {
      status = Incomplete;
      break;
    }
 }

  if(status == Success)
  {
    m_length = length;
    m_result = result;
  }
  else
  {
    m_length = 0;
    m_result = -1;
  }
  return status;
}
