/*
 * Copyright (C) 2004   Derrick Moser
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include "Composer.h"

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <syscfg.h>
#include <iml/SunIM.h>

static void *salloc(iml_session_t *s, int len)
{
  return s->If->m->iml_new(s, len);
}

static IMText *create_text(iml_session_t *s, int *str, int len, bool underline)
{
  IMText *t = (IMText *)salloc(s, sizeof(IMText));
  if(t)
  {
    UTFCHAR *c = (UTFCHAR *)salloc(s, sizeof(UTFCHAR) * len);
    IMFeedbackList *fl = (IMFeedbackList *)salloc(s, sizeof(IMFeedbackList) * len);
    for(int i = 0; i < len; ++i)
    {
      c[i] = str[i];
      fl[i].count_feedbacks = 1;
      IMFeedback *f = (IMFeedback *)salloc(s, sizeof(IMFeedback));
      IM_FEEDBACK_TYPE(f) = IM_DECORATION_FEEDBACK;
      IM_FEEDBACK_VALUE(f) = (underline ? IMUnderline : IMNormal);
      fl[i].feedbacks = f;
    }
    t->text.utf_chars = c;
    t->feedback = fl;
  }
  t->encoding = UTF16_CODESET;
  t->char_length = len;
  t->count_annotations = 0;
  t->annotations = 0;
  return t;
}


class SimpleHangulComposer : public Composer
{
  bool m_display_preedit;
  iml_session_t *m_session;

public:
  SimpleHangulComposer(iml_session_t *s);

  void showStatus(iml_inst **list);
};

SimpleHangulComposer::SimpleHangulComposer(iml_session_t *s) : m_session(s), m_display_preedit(false)
{
}

void SimpleHangulComposer::showStatus(iml_inst **list)
{
  iml_inst *lp = 0;
  int len = m_output.getSize();
  if(len)
  {
    IMText *text = create_text(m_session, m_output.getBuffer(), len, false);
    m_output.clear();
    lp = m_session->If->m->iml_make_commit_inst(m_session, text);
    m_session->If->m->iml_link_inst_tail(list, lp);
  }

  if(m_display_preedit)
  {
    lp = m_session->If->m->iml_make_preedit_erase_inst(m_session);
    m_session->If->m->iml_link_inst_tail(list, lp);
    m_display_preedit = false;
  }
  len = m_input.getSize();
  if(len)
  {
    IMText *text = create_text(m_session, m_input.getBuffer(), len, true);
    lp = m_session->If->m->iml_make_preedit_draw_inst(m_session, text);
    m_display_preedit = true;
    m_session->If->m->iml_link_inst_tail(list, lp);
  }
}


static Bool if_simplehangul_OpenIF(iml_if_t *If)
{
  return True;
}

static Bool if_simplehangul_CloseIF(iml_if_t *If)
{
  return True;
}

static Bool if_simplehangul_GetIFValues(iml_if_t *If, IMArgList args, int num_args)
{
  return True;
}

static Bool if_simplehangul_SetIFValues(iml_if_t *If, IMArgList args, int num_args)
{
  return True;
}

static Bool if_simplehangul_OpenDesktop(iml_desktop_t *desktop, IMArgList args, int num_args)
{
  return True;
}

static Bool if_simplehangul_CloseDesktop(iml_desktop_t *desktop)
{
  return True;
}

static Bool if_simplehangul_CreateSC(iml_session_t *s, IMArgList args, int num_args)
{
  s->specific_data = new SimpleHangulComposer(s);
  return True;
}

static Bool if_simplehangul_DestroySC(iml_session_t * s)
{
  delete (SimpleHangulComposer *)s->specific_data;
  return True;
}

static Bool if_simplehangul_GetSCValues(iml_session_t * s, IMArgList args, int num_args)
{
  return True;
}

static Bool if_simplehangul_SetSCValues(iml_session_t *s, IMArgList args, int num_args)
{
  for(int i = 0; i < num_args; i++, args++)
  {
    switch(args->id)
    {
      case SC_TRIGGER_ON_NOTIFY:
          {
            static int hangul[] =
            {
              '[', ' ', 0xac00, ' ', ']'
            };

            iml_inst *list = 0;
            iml_inst *lp;

            lp = s->If->m->iml_make_status_start_inst(s);
            s->If->m->iml_link_inst_tail(&list, lp);
            IMText *text = create_text(s, hangul, sizeof(hangul)/sizeof(int), false);
            lp = s->If->m->iml_make_status_draw_inst(s, text);
            s->If->m->iml_link_inst_tail(&list, lp);
            lp = s->If->m->iml_make_preedit_start_inst(s);
            s->If->m->iml_link_inst_tail(&list, lp);
            SimpleHangulComposer *composer = (SimpleHangulComposer *)s->specific_data;
            if(composer)
              composer->showStatus(&list);
            s->If->m->iml_execute(s, &list);
          }
          break;

      case SC_TRIGGER_OFF_NOTIFY:
          {
            iml_inst *list = 0;
            iml_inst *lp;

            SimpleHangulComposer *composer = (SimpleHangulComposer *)s->specific_data;
            if(composer)
            {
              composer->clear();
              composer->showStatus(&list);
            }
            lp = s->If->m->iml_make_preedit_done_inst(s);
            s->If->m->iml_link_inst_tail(&list, lp);
            IMText *text = create_text(s, 0, 0, false);
            lp = s->If->m->iml_make_status_draw_inst(s, text);
            s->If->m->iml_link_inst_tail(&list, lp);
            lp = s->If->m->iml_make_status_done_inst(s);
            s->If->m->iml_link_inst_tail(&list, lp);
            s->If->m->iml_execute(s, &list);
          }
          break;
    }
  }
  return True;
}

static IMText *if_simplehangul_ResetSC(iml_session_t *s)
{
  return 0;
}

static void if_simplehangul_SetSCFocus(iml_session_t *s)
{
}

static void if_simplehangul_UnsetSCFocus(iml_session_t *s)
{
}

static void if_simplehangul_SendEvent(iml_session_t *s, IMInputEvent *ev)
{
  if(ev->type == IM_EventKeyList)
  {
    IMKeyListEvent *keylistevent = (IMKeyListEvent *)ev;
    IMKeyEventStruct *key = (IMKeyEventStruct *)keylistevent->keylist;
    SimpleHangulComposer *composer = (SimpleHangulComposer *)s->specific_data;
    if(composer)
    {
      iml_inst *list = 0;

      if(key->keyCode == IM_VK_BACK_SPACE && !composer->isEmpty())
        composer->backspace();
      else if(key->keyCode == IM_VK_SPACE && !composer->isEmpty())
        composer->convert();
      else if(key->keyCode == IM_VK_ENTER && !composer->isEmpty())
        composer->accept();
      else if(key->keyChar)
        composer->append(key->keyChar);
      else if(composer->isEmpty())
      {
        iml_inst *lp = s->If->m->iml_make_keypress_inst(s, key);
        s->If->m->iml_link_inst_tail(&list, lp);
      }
      composer->showStatus(&list);
      s->If->m->iml_execute(s, &list);
    }
  }
}

static if_methods_t if_methods =
{
  if_simplehangul_OpenIF,
  if_simplehangul_CloseIF,
  if_simplehangul_GetIFValues,
  if_simplehangul_SetIFValues,
  if_simplehangul_OpenDesktop,
  if_simplehangul_CloseDesktop,
  if_simplehangul_CreateSC,
  if_simplehangul_DestroySC,
  if_simplehangul_GetSCValues,
  if_simplehangul_SetSCValues,
  if_simplehangul_ResetSC,
  if_simplehangul_SetSCFocus,
  if_simplehangul_UnsetSCFocus,
  if_simplehangul_SendEvent
};

static UTFCHAR language_engine_name[] =
{
  0xac04, 0xb2e8, 0xd55c, ' ', 0xd55c, 0xae00, 0
};

static UTFCHAR language_name[] =
{
  0xd55c, 0xad6d, 0xc5b4, 0
};

static IMLEName im_language_engine_name =
{
  "simplehangul",
  language_engine_name
};

static IMLocale locales[] =
{
  { "ko", language_name },
  { "ko_KR", language_name },
  { 0, 0 }
};

extern "C" void if_GetIfInfo(IMArgList args, int num_args)
{
  for(int i = 0; i < num_args; ++i, ++args)
  {
    switch(args->id)
    {
      case IF_VERSION:
          args->value = (IMArgVal)PACKAGE_VERSION;
          break;

      case IF_METHOD_TABLE:
          args->value = (IMArgVal)&if_methods;
          break;

      case IF_LE_NAME:
          args->value = (IMArgVal)&im_language_engine_name;
          break;

      case IF_SUPPORTED_LOCALES:
          args->value = (IMArgVal)&locales;
          break;

      case IF_NEED_THREAD_LOCK:
          args->value = (IMArgVal)False;
          break;
    }
  }
}
